/**
 * The <code>Actor</code> class is used to get an abstraction of the
 * plain text commands so that the can be used via methods. The methods
 * compose the commands and send them via the communicationChannel link.
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.1
 */
public class Actor
{
    /** reference to the communicationChannel */
    private CommunicationChannel communicationChannel;

    /**
     * Creates a new <code>Actor</code> instance.
     *
     * @param communicationChannel a <code>CommunicationChannel</code> value defining the
     * communicationChannel link to the server via which the commands can be
     * sent
     * @see CommunicationChannel
     */
    public Actor(CommunicationChannel communicationChannel)
    {
        this.communicationChannel = communicationChannel;
    }

    /**
     * Registers itself to to the server specifying protocol version 7.0.<BR>
     * NOTE: If you don't use version 7.0, it may occur that you will not receive
     * a sense_body event and that the names of the objects will be written in a different
     * way like "flag" instead of "f". This may cause problems if you assume a special 
     * format for parsing.
     *
     * @param teamName The name of the team
     * the player should join or create if it does not already exist and there
     * less than two teams already present on the field
     */
    public void init(String teamName)
    {
        communicationChannel.send(new String("(init " + teamName + "(version 7.0))"));
    }

     /**
     * Receives incoming messages from the server. They
     * are automatically classified and passed on to the correct parsing/representation
     * classes. <BR>
     * NOTE: The <code>receive()</code> method is blocking because is uses the blocking
     * <code>receive()</code> method of the <code>CommunicationChannel</code> class. Therefor
     * the thread will be interrupted until a new message arrives.
     * @see CommunicationChannel
     *
     * @return an <code>Event</code> object containing the parsed information
     * of the read message
     */
    public Event event()
    {
        String msg;
        Event event = null;

        while (event == null) {
            // receive server message - block until one arrives
            msg = communicationChannel.receive();

            // it's a visual ("see") or a body ("sense body") event
            if (msg.startsWith("(see")) {
                // it's a visual ("see") event
                event = new VisualEvent(msg);
            }
            else if (msg.startsWith("(sense_body")) {
                // it's a body ("sense_body") event
                event = new BodyEvent();
            }
        }

        return event;
    }

    /**
     * Gives a running impulse
     *
     * @param power a <code>double</code> value containing the power of the
     * impulse
     */
    public void dash(double power)
    {
        communicationChannel.send(new String("(dash " + power + ")"));
    }

    /**
     * Kicks into a given direction with given power
     *
     * @param power a <code>double</code> value specifying the power of the kick
     * @param direction a <code>double</code> value giving the direction of the kick
     */
    public void kick(double power, double direction)
    {
        communicationChannel.send(new String("(kick " + power + " " + direction + ")"));
    }

    /**
     * Places the player to a given position on the field. Only possible:
     * <ul>
     * <li> before kick off
     * <li> after a goal
     * </ul>
     *
     * @param x gives the position in x direction.
     * The position can vary from -52.5 to 52.5. The unit is meter.
     * Negative values stand for the own half of the field, positive values stand for
     * the opponent's half. The value itself gives the distance from the mid-line.
     * @param y gives the position in y direction.
     * The position can vary from -34.0 to 34.0. The unit is meter.
     * Negative values stand for the left side of the field when looked in direction of
     * the opponent's goal.Positive value stand for the right side.
     * The value itself gives the distance from the line going through the center of both
     * goal and the middle point of the field.
     */
    public void move(double x, double y)
    {
        communicationChannel.send(new String("(move " + x + " " + y + ")"));
    }

    /**
     * Turns the body of the client on the field.
     *
     * @param moment is the angle in degree you want to turn.
     * The orientation is clockwise so that positive angles mean right turns. The absolute
     * value of the angle must be smaller than 180 degrees. Otherwise the the server will
     * take the turn as a 180 degree turn. So it should be assured that the angle are
     * normalized before usage
     */
    public void turn(double moment)
    {
        communicationChannel.send(new String("(turn " + moment + ")"));
    }
}
