import java.io.*;
import java.net.*;


/**
 * CommunicationChannel
 *
 * The class handles UDP network traffic from/to the rcssserver.
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.1
 */
public class CommunicationChannel
{
    /** socket for incoming, outgoing messages */
    private DatagramSocket socket;
    /** IP address of the server */
    InetAddress serverAddress;
    /** port number of the server */
    private int serverPort;

    /**
     * Creates a new <code>CommunicationChannel</code> instance.
     *
     * @param serverName The address of the server specified by its name or its IP address
     * @param serverPort The port of the server
     */
    public CommunicationChannel(String serverName, int serverPort) {
        try {
            // construct datagram socket and
            // bind it to any available port on the local machine
            this.socket        = new DatagramSocket();
            // set target address
            this.serverAddress = InetAddress.getByName(serverName);
            // set target port
            this.serverPort    = serverPort;
        } catch (SocketException se) {
            System.err.println("CommunicationChannel: Could not create datagram socket.");
            System.exit(1);
        } catch (UnknownHostException uhe) {
            System.err.println("CommunicationChannel: Could not determine the server's IP address.");
            System.exit(1);
        }

    }


    /**
     * Sends a <code>String</code> to the server
     *
     * @param message a <code>String</code> value containing the message which
     * has to be sent to the server.
     */
    public void send(String message) {
        try {
            // convert the message to a byte array
            byte[] buffer = message.getBytes();
            // create the packet from the buffer and set the server's IP address and port
            DatagramPacket packet =
                new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
            // send the packet
            socket.send(packet);
        } catch (IOException ioe) {
            System.err.println("CommunicationChannel: Could not send datagram packet.");
        }
    }


    /**
     * Receives a message from the server
     *
     * @return a <code>String</code> containing the message read from the socket
     */
    public String receive() {
        // allocate buffer for server message
        byte[] buffer = new byte[1024];
        // create datagram packet for receiving
        DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
        try {
            // receive packet, the method blocks until a datagram arrives
            socket.receive(packet);
        } catch (IOException ioe) {
            System.err.println("CommmunicationChannel: Could not receive datagram packet.");
        }
        // get the server port for the following actions
        serverPort = packet.getPort();
        // return the message as string
        return new String(packet.getData()).substring(0, packet.getLength()-1)+" ";  // cut off null characters & \n in buffer
    }
}

