import java.util.*;
/**
 * This is the central class of the Simpleclient. Here, all visual information
 * is evaluated and converted into decisions resp. actions to:
 * <ul>
 * <li> find out the positions of the ball and the goal
 * <li> turn towards the ball
 * <li> run to the ball
 * <li> kick it into the direction of the right goal
 * </ul>
 *
 * So one limitation is, that the client only can play from left to right
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.1
 */
public class Player extends Thread
{

    /** the actor module of the client */
    private Actor  actor;
    /** the name of the team (for initialization) */
    private String teamName;

    /** constant defining the START status */
    final private int START         = 0;
    /** constant defining the PLAYING status */
    final private int PLAYING       = 1;

    /** current status the player is in; can be <code>START</code> or <code>PLAYING</code>
     * @see Player#START
     * @see Player#PLAYING */
    private int status = START;

    /** the current information regarding the ball*/
    private ObjectPerception theBall = null;
    /** the current information regarding the goal*/
    private ObjectPerception theGoal = null;

    /** the actual timestamp of last visual event received */
    private int time = -1;


    /**
     * Creates a new <code>Player</code> instance.
     *
     * @param servername a <code>String</code> value specifying the name of the server
     * @param serverport an <code>int</code> value specifying the port on which the server
     *                   listens for incoming packets
     * @param teamName a <code>String</code> value is the name of the team you want to join/create
     */
    public Player(String servername, int serverport, String teamName)
    {
        this.teamName = teamName;

        CommunicationChannel communication = new CommunicationChannel(servername, serverport);
        actor   = new Actor(communication);
    }


    /**
     * This method overwrites <code>run()</code> from <code>Thread</code>. It is
     * executed when <code>start()</code> is called for the <code>Player</code>
     * Thread
     *
     */
    public void run()
    {
        Event event = null;

        // register player
        actor.init(teamName);

        // and play
        while (true) {
            // receive event - block until one arrives
            event = actor.event();

            // update information about the world
            if (event.isVisualEvent())
            {
              for (Enumeration e=((VisualEvent)event).elements(); e.hasMoreElements();) {
                // check for the ball and the oponents' goal
                ObjectPerception objectPerception = (ObjectPerception)e.nextElement();

                if (objectPerception.name().startsWith("b")) {
                    theBall = objectPerception;
                    System.out.println("Found ball");
                }
                else if (objectPerception.name().startsWith("g r")) {
                    theGoal = objectPerception;
                    System.out.println("Found goal");
                }
              }
            }
            else if (event.isBodyEvent()) { // only act upon receipt of a body sense event
              play();
            }
        }
    }

    /**
     * Processes the information gained and transforms it into a decision/action.
     * It should only be called once per 100ms period, so its proper execution on
     * the server is assured.<BR>
     * NOTE: "Big" commands like the "dash"-command consume 100ms while "small" 
     * commands like the one to turn the neck of the player consume less time. So 
     * it might be possible to send more commands. As we use only the longer ones
     * this fact is irrelevant for our client, but might be an issue in the
     * development of more sophisticated movements
     *
     */
    private void play()
    {
      switch (status) {
        case START:
            // set the player to a random position DEEP in the left field
            actor.move(-Math.random()*22.5 - 30, Math.random()*2*20.0 - 20.0);
            status = PLAYING;
            break;

        case PLAYING:
            // already found the goal?
            if (theGoal == null) {
                // no, look around
                actor.turn(40);
            }
            else if (theBall == null) {  // already found the ball?
                // no, look around
                actor.turn(40);
            } else {
                // we found the ball
                // turn to it if necessary but only if new ball info is available
                if (Math.abs(theBall.direction()) > 1.0 && theBall.time()!=time) {
                    actor.turn(theBall.direction());
                    // remember the time of the info the action was based on
                    System.out.println("Turning towards ball");
                    time=theBall.time();
                // check the distance
                } else if (theBall.distance() > 0.5) {
                    actor.dash(10 * theBall.distance()); // get slower as we approach the ball
                    System.out.println("Running towards ball");
                // kick
                } else {
                    actor.kick(50, theGoal.direction());
                }
            }
            break;
        }
    }

    /**
     * Starts the client
     *
     * @param args The arguments passed must specify <code>SERVERNAME PORT TEAMNAME</code> in this order.
     */
    public static void main (String[] args) {
        if (args.length != 3) {
            System.out.println("Usage : java Player SERVERNAME PORT TEAMNAME");
            System.exit(1);
        }

        Player player = new Player(args[0], Integer.parseInt(args[1]), args[2]);
        //Player player = new Player("localhost", 6000, "Brain");
        player.start();
    }
}
