import java.util.*;

/**
 * This class parses all messages regarding visual information of the game,
 * i.e. the <code>see</code> messages, and represents the information in
 * a list of <code>ObjectInfo</code>
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.1
 */
public class VisualEvent extends Event
{
    /**
     * List for the objects contained in an event
     * e.g. a see message contains flags, players, the ball etc.
     * @see ObjectPerception
     *
     */
    private Vector objectPerceptionList = new Vector();

    /**
     * return the perceived objects
     *
     * @return an <code>Enumeration</code> containing the perceived and processed 
     * <code>ObjectPerception</code> objects
     * @see ObjectPerception
     */
    public Enumeration elements() {return objectPerceptionList.elements();}

    /**
     * adds an element to the list of perceived objects to the internal list
     *
     * @param objectPerception an <code>ObjectPerception</code> value containing the processed data of the object
     */
    public void add(ObjectPerception objectPerception) {objectPerceptionList.addElement(objectPerception);}

    /**
     * indicates that the event is of type <code>VisualEvent</code>
     *
     * @return a <code>boolean</code> value indicating that this event is a
     * <code>VisualEvent</code>
     */
    public boolean isVisualEvent() {return true;}


    /**
     * Creates a new <code>VisualEvent</code> instance, extracts the information contained
     * and stores it in a list of <code>ObjectPerception</code>
     * @see ObjectPerception
     *
     * @param message a <code>String</code> value which contains the
     * <code>see</code> message of the <code>rcssserver</code> to be
     * parsed and stored in the instance
     */
    public VisualEvent(String message)
    {
        // use parantheses as string delimiters
        StringTokenizer	tokenizer = new StringTokenizer(message, "()");

        // extract the time stamp
        // - nextToken returns something like "see 15 "
        // - use substring and trim whitespaces
        // Example:
        // The received String is:
        // (see 0 ((g r) 66.7 34) ((b) 20.3 5))

        // The first Token extracted would be:
        // "see 0 " (the blank at the end is included)
        // substring(4) returns "0 "; trim() removes leading and trailing blanks
        // so it returns "0"

        int time = Integer.parseInt(tokenizer.nextToken().substring(4).trim());

        // process all seen objects
        while (tokenizer.hasMoreTokens())
        {
          // get the object name
          // in the example the next Token would be:
          // "g r" for the right goal
          String code=tokenizer.nextToken();

          // sometimes see messages arrive without any objects following
          if (!code.equals(" "))
          {

            // get distance and direction
            String data = tokenizer.nextToken();

            // skip to next object
            tokenizer.nextToken();

            switch (code.charAt(0)) {
              // we only handle ball and goal objects ...
              case 'b':
              case 'g':
                // in the example variable data would contain:
                // " 66.7 34"
                // to parse this, we use a new Tokenizer taking space as a delimiter
                StringTokenizer st   = new StringTokenizer(data, " ");
                double distance  = Double.valueOf(st.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());
                double direction = Double.valueOf(st.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());

                // create a new object info
                ObjectPerception objectPerception = new ObjectPerception(time, code, distance, direction);
                this.add(objectPerception);
                break;

              // not handled yet
              case 'p':
              case 'f':
              case 'l':
                break;
            }
          }
        }
    }
}
